#!/usr/bin/env python

import os
import sys
import getopt
import exceptions

if not os.environ.has_key( "REPET_PATH" ):
    print "*** Error: no environment variable REPET_PATH"
    sys.exit(1)
sys.path.append( os.environ.get( "REPET_PATH" ) )
import pyRepet.seq.fastaDB

#------------------------------------------------------------------------------

def help():

    """
    Give the list of the command-line options.
    """

    print ""
    print "usage: %s [ options ]" % ( sys.argv[0] )
    print "options:"
    print "     -h: this help"
    print "     -i: name of the input file (format='fasta')"
    print "     -n: number of sequences per output file (default=1)"
    print "     -d: record the output fasta files in a directory called 'batches'"
    print "     -v: verbose (default=0/1)"
    print ""

#------------------------------------------------------------------------------

def main():

    """
    This program splits a fasta file containing several sequences.
    """

    inFileName = ""
    nbSeq = 1
    newDir = False
    verbose = 0

    try:
        opts, args = getopt.getopt(sys.argv[1:],"hi:n:dv:")
    except getopt.GetoptError:
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
            sys.exit(0)
        elif o == "-i":
            inFileName = a
        elif o == "-n":
            nbSeq = int(a)
        elif o == "-d":
            newDir = True
        elif o == "-v":
            verbose = int(a)

    if inFileName == "":
        print "*** Error: missing input file"
        help()
        sys.exit(1)

    if verbose > 0:
        print "START %s" % ( sys.argv[0].split("/")[-1] )
        sys.stdout.flush()

    pyRepet.seq.fastaDB.dbSplit( inFileName, nbSeq, newDir, verbose )

    if verbose > 0:
        print "END %s" % ( sys.argv[0].split("/")[-1] )
        sys.stdout.flush()

    return 0

#------------------------------------------------------------------------------

if __name__ == "__main__":
    main()
