#!/usr/bin/env python

import user, os, sys, getopt, string

if not os.environ.has_key( "REPET_PATH" ):
    print "*** Error: no environment variable REPET_PATH"
    sys.exit(1)
sys.path.append( os.environ.get( "REPET_PATH" ) )

import pyRepet.seq.fastaDB

#------------------------------------------------------------------------------

def help():

    print
    print "usage: ",sys.argv[0],"[ options ]"
    print "options:"
    print "     -h: this help"
    print "     -i: name of the input file (format='fasta')"
    print "     -o: name of the output fasta file (default=inFileName+'.filtered')"
    print "     -S: filter the consensus classified as 'SSR'"
    print "     -s: length below which a SSR is filtered (e.g. 300, default=0)"
    print "     -g: filter the consensus classified as 'HostGene'"
    print "     -c: filter the consensus classified as 'confused'"
    print "     -N: filter the consensus classified as 'NoCat'"
    print "     -n: minimum number of sequences in the MSA from which the NoCat consensus has been built (default=3)"
    print "     -v: verbose (default=0/1/2)"
    print

#------------------------------------------------------------------------------

def main():

    """
    This program filters the consensus classified by 'TEclassifier.py'.
    """

    inFaFileName = ""
    outFaFileName = ""
    filterSSRs = False
    maxLengthToFilterSSRs = 0
    filterHostGenes = False
    filterConfused = False
    filterNoCat = False
    nbAlignSeqNoCat = 3
    verbose = 0

    try:
        opts, args= getopt.getopt( sys.argv[1:], "hi:o:Ss:gcNn:v:" )
    except getopt.GetoptError, err:
        print str(err)
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
            sys.exit(0)
        elif o == "-i":
            inFaFileName = a
        elif o == "-o":
            outFaFileName = a
        elif o == "-S":
            filterSSRs = True
        elif o == "-s":
            maxLengthToFilterSSRs = int(a)
        elif o == "-g":
            filterHostGenes = True
        elif o == "-c":
            filterConfused = True
        elif o == "-N":
            filterNoCat = True
        elif o == "-n":
            nbAlignSeqNoCat = int(a)
        elif o == "-v":
            verbose = int(a)

    if inFaFileName == "":
        print "*** Error: missing input file name"
        help()
        sys.exit(1)

    if verbose > 0:
        print "\nbeginning of %s" % ( sys.argv[0].split("/")[-1] )
        sys.stdout.flush()

    pyRepet.seq.fastaDB.filterClassifConsensus( inFaFileName, outFaFileName, filterSSRs, maxLengthToFilterSSRs, filterHostGenes, filterConfused, filterNoCat, nbAlignSeqNoCat, verbose )

    if verbose > 0:
        print "%s finished successfully\n" % ( sys.argv[0].split("/")[-1] )
        sys.stdout.flush()

    return 0

#------------------------------------------------------------------------------

if __name__ == '__main__':
    main()
