#!/usr/bin/env python

import user, os, sys, getopt, exceptions

#------------------------------------------------------------------------------

def help():

    """
    Give the list of the command-line options.
    """

    print ""
    print "usage: ",sys.argv[0],"[ options ]"
    print "options:"
    print "     -h: this help"
    print "     -i: name of the input file (format='align')"
    print "     -o: name of the output file (default=inFileName+'.filtered')"
    print "     -E: maximum E-value (default=1e-300)"
    print "     -l: minimum length (default=100)"
    print "     -I: minimum identity (default=90)"
    print "     -L: maximum length (default=20000)"
    print "     -v: verbose (default=0/1)"
    print ""

#------------------------------------------------------------------------------

def main():

    """
    This program filters the output from BLASTER ('align' file recording HSPs).
    """

    inFileName = ""
    outFileName = ""
    maxEValue = 1e-300
    minIdentity = 90
    minLength = 100
    maxLength = 20000
    verbose = 0

    try:
        opts,args=getopt.getopt(sys.argv[1:],"hi:o:E:l:I:L:v:")
    except getopt.GetoptError:
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
            sys.exit()
        elif o == "-i":
            inFileName = a
        elif o == "-o":
            outFileName = a
        elif o == "-E":
            maxEValue = float(a)
        elif o == "-l":
            minLength = int(a)
        elif o == "-I":
            minIdentity = int(a)
        elif o == "-L":
            maxLength = int(a)
        elif o == "-v":
            verbose = int(a)

    if inFileName == "":
        print "*** Error: missing input file name"
        help()
        sys.exit(1)

    if verbose > 0:
        print "\nbeginning of %s" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    inFile = open( inFileName, "r" )

    if outFileName == "":
        outFileName = "%s.filtered" % ( inFileName )
    outFile = open( outFileName, "w" )

    line = inFile.readline()
    while True:
        if line == "":
            break
        data = line.split("\t")
        qryName = data[0]
        qryStart = data[1]
        qryEnd = data[2]
        sbjName = data[3]
        sbjStart = data[4]
        sbjEnd = data[5]
        Evalue = data[6]
        score = data[7]
        identity = data[8]

        if int(qryStart) < int(qryEnd):
            matchLength = int(qryEnd) - int(qryStart) + 1
        elif int(qryStart) > int(qryEnd):
            matchLength = int(qryStart) - int(qryEnd) + 1

        if float(Evalue) <= maxEValue and matchLength >= minLength and \
               float(identity) >= minIdentity and matchLength <= maxLength:
            string = qryName + "\t" + qryStart + "\t" + qryEnd + "\t" +\
                     sbjName + "\t" + sbjStart + "\t" + sbjEnd + "\t" +\
                     Evalue + "\t" + score + "\t" + identity
            outFile.write( string )
        else:
            string = "qry %s (%s-%s) vs subj %s (%s-%s): Eval=%s, identity=%s, matchLength=%s" %\
                     ( qryName, qryStart, qryEnd, sbjName, sbjStart, sbjEnd, Evalue, identity.split("\n")[0], matchLength )
            if verbose > 0:
                print string; sys.stdout.flush()

        line = inFile.readline()

    inFile.close()
    outFile.close()

    if verbose > 0:
        print "%s finished successfully\n" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    return 0

#------------------------------------------------------------------------------

if __name__ == '__main__':
    main()
