#!/usr/bin/env python

import user, os, sys, getopt, exceptions

if not os.environ.has_key( "REPET_PATH" ):
    print "*** Error: no environment variable REPET_PATH"
    sys.exit(1)
sys.path.append( os.environ["REPET_PATH"] )

from pyRepet.parser.Parser import *
from pyRepet.launcher.programLauncher import *
from pyRepet.seq.BioseqDB import *

#------------------------------------------------------------------------------

def help():

    """
    Give the list of command-line options.
    """

    print ""
    print "usage:",sys.argv[0],"[ options ]"
    print "options:"
    print "     -h: this help"
    print "     -p: project name (default=alignFileName)"
    print "     -f: name of the file used as query by Blaster (format='fasta')"
    print "     -a: name of the file returned by Blaster (format='align')"
    print "     -c: clean"
    print "     -v: verbose (default=0/1)"
    print ""

#------------------------------------------------------------------------------

def main():

    """
    This programs launches Recon by giving the same files than to Grouper.
    """

    projectName = ""
    faFileName = ""
    alignFileName = ""
    clean = False
    verbose = 0

    try:
        options,arguments = getopt.getopt(sys.argv[1:],"hp:f:a:cv:")
    except getopt.GetoptError, err:
        print str(err)
        help()
        sys.exit(1)
    for o,a in options:
        if o == "-h":
            help()
            sys.exit(0)
        elif o == "-p":
            projectName = a
        elif o == "-f":
            faFileName = a
        elif o == "-a":
            alignFileName = a
        elif o == "-c":
            clean = True
        elif o == "-v":
            verbose = int(a)

    if faFileName == "" or alignFileName == "":
        print "*** Error: compulsory options are missing"
        help()
        sys.exit(1)

    if verbose > 0:
        print "\nbeginning of %s\n" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    if projectName == "":
        projectName = alignFileName

    #--------------------------------------------------------------------------

    # prepare the MSP file, like this:
    # score %iden start end query_name start end subject_name \n

    if verbose > 0:
        print "prepare MSP file"; sys.stdout.flush()
    if os.path.exists( os.environ["REPET_PATH"] + "/bin/align2recon" ):
        prg = os.environ["REPET_PATH"] + "/bin/align2recon"
        cmd = prg
        cmd += " -i %s" % ( alignFileName )
        cmd += " -o %s_MSP_file" % ( projectName )
        cmd += " -v %i" % ( verbose - 1 )
        log = os.system( cmd )
        if log != 0:
            print "*** Error: %s returned %i" % ( prg, log )
            sys.exit(1)
    else:
        parser = Parser( alignFileName, projectName + "_MSP_file" )
        parser.align2recon()

    #--------------------------------------------------------------------------

    # prepare the seq file

    if verbose > 0:
        print "prepare seq file"; sys.stdout.flush()
    bsDB = BioseqDB( faFileName )
    lSeqNames = bsDB.idx.keys()
    seqListFile = open( projectName + "_seq_list", "w" )
    seqListFile.write( str(len(lSeqNames)) + "\n" )
    lSeqNames.sort()
    for seqName in lSeqNames:
        seqListFile.write( ">%s\n" % ( seqName ) )
    seqListFile.close()

    #--------------------------------------------------------------------------

    if verbose > 0:
        print "Recon is running..."; sys.stdout.flush()
    pL = programLauncher()
    pL.launchRecon( projectName + "_seq_list", projectName + "_MSP_file" )

    if clean == True:
        os.system( "rm -rf edge_redef_res ele_def_res ele_redef_res" )

    #--------------------------------------------------------------------------

    if verbose > 0:
        print "convert results into 'map' format"; sys.stdout.flush()
    parser = Parser( "summary/eles", projectName + "_Recon.map" )
    parser.recon2map()

    if clean == True:
        os.system( "rm -rf summary images" )

    #--------------------------------------------------------------------------

    if verbose > 0:
        print "generate fasta from map"; sys.stdout.flush()
    prg = os.environ["REPET_PATH"] + "/bin/map2db"
    cmd = prg
    cmd += " " + projectName + "_Recon.map"
    cmd += " " + faFileName
    log = os.system( cmd )
    if log != 0:
        print "*** Error: %s returned %i" % ( prg, log )
        sys.exit(1)

    os.system( "mv " + projectName + "_Recon.map.flank_size0.fa " + projectName + "_Recon.fa" )

    #--------------------------------------------------------------------------

    if verbose > 0:
        print "\n%s finished successfully\n" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    return 0

#------------------------------------------------------------------------------

if __name__ == '__main__':
    main()
