#!/usr/bin/env python

import user, os, sys, getopt, exceptions

if not os.environ.has_key( "REPET_PATH" ):
    print "*** Error: no environment variable REPET_PATH"
    sys.exit(1)
sys.path.append( os.environ["REPET_PATH"] )

import pyRepet.seq.fastaDB

#-----------------------------------------------------------------------------

def help():

    """
    Give the list of the command-line options.
    """

    print ""
    print "usage:",sys.argv[0]," [ options ]"
    print "option:"
    print "    -h: this help"
    print "    -i: input fasta file"
    print "    -c: clustering method (Grouper/Recon/Piler)"
    print "    -d: create a directory for each group"
    print "    -H: simplify sequence headers"
    print "    -v: verbose (default=0/1/2)"
    print ""

#-----------------------------------------------------------------------------

def main():

    """
    This program records all the sequences belonging to the same group in the same fasta file
    """

    inFileName = ""
    clusteringMethod = ""
    createDir = False
    simplifyHeader = False
    verbose = 0

    try:
        opts, args = getopt.getopt(sys.argv[1:],"hi:c:dHv:")
    except getopt.GetoptError, err:
        print str(err)
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
            sys.exit(0)
        elif o == "-i":
            inFileName = a
        elif o == "-c":
            clusteringMethod = a
        elif o == "-d":
            createDir = True
        elif o == "-H":
            simplifyHeader = True
        elif o == "-v":
            verbose = int(a)

    if  inFileName == "" or clusteringMethod == "":
        print "*** Error: missing compulsory options"
        help()
        sys.exit(1)

    if verbose > 0:
        print "\nbeginning of %s\n" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    pyRepet.seq.fastaDB.splitSeqPerCluster( inFileName, clusteringMethod, simplifyHeader, createDir )

    if verbose > 0:
        print "%s finished successfully\n" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    return 0

#-----------------------------------------------------------------------------

if __name__ == '__main__':
    main()
