#!/usr/bin/env python

import user, os, sys, getopt, string, ConfigParser, logging
from copy import *

if not os.environ.has_key( "REPET_PATH" ):
    print "*** Error: no environment variable REPET_PATH"
    sys.exit(1)
sys.path.append( os.environ["REPET_PATH"] )
import pyRepet.sql.RepetDB
import pyRepet.sql.TableAdaptator
import pyRepet.coord.Map
import pyRepet.coord.Path
import pyRepet.coord.Match
import pyRepet.coord.Set

#-----------------------------------------------------------------------------

def help():

    """
    Give the list of the command-line options.
    """

    print ""
    print "usage:",sys.argv[0],"[options]"
    print "options:"
    print "     -h: this help"
    print "     -i: name of the input table (format=path)"
    print "     -o: name of the output table (default=inTable+'_nr')"
    print "     -C: configuration file from TEdenovo or TEannot pipeline"
    print "     -H: MySQL host (if no configuration file)"
    print "     -U: MySQL user (if no configuration file)"
    print "     -P: MySQL password (if no configuration file)"
    print "     -D: MySQL database (if no configuration file)"
    print "     -v: verbose (default=0/1/2)"
    print ""

#-----------------------------------------------------------------------------

def merge_set_table( db, table ):

    merged_table=table+"_merged"
    db.remove_if_exist(merged_table)
    sql_cmd="CREATE TABLE %s ( path int unsigned, name varchar(255), chr varchar(255), start int, end int);"% (merged_table)
    db.execute(sql_cmd)

    mtableAdaptator=pyRepet.sql.TableAdaptator.TableSetAdaptator(db,merged_table)
    tableAdaptator=pyRepet.sql.TableAdaptator.TableSetAdaptator(db,table)
    contigs=tableAdaptator.getContig_name()

    for c in contigs:
        clist=tableAdaptator.getSetList_from_contig(c)
        qhash=pyRepet.coord.Set.set_list_split(clist)
        for k,alist in qhash.items():
            alist=pyRepet.coord.Set.set_list_merge(alist)
            mtableAdaptator.insSetList(alist)

    return merged_table

#-----------------------------------------------------------------------------

def main():

    """
    This program removes doublons within a path table.
    """

    inTable = ""
    outTable = ""
    configFileName = ""
    host = ""
    user = ""
    passwd = ""
    dbname = ""
    verbose = 0

    try:
        opts, args = getopt.getopt(sys.argv[1:],"hi:o:C:H:U:P:D:v:")
    except getopt.GetoptError, err:
        print str(err)
        help()
        sys.exit(1)
    for o,a in opts:
        if o == "-h":
            help()
            sys.exit(0)
        elif o == "-i":
            inTable = a
        elif o == "-o":
            outTable = a
        elif o == "-C":
            configFileName = a
        elif o == "-H":
            host = a
        elif o == "-U":
            user = a 
        elif o == "-P":
            passwd = a
        elif o == "-D":
            dbname = a
        elif o == "-v":
            verbose = int(a)

    if inTable == "":
        print "*** Error: missing input table name"
        help()
        sys.exit(1)

    if verbose > 0:
        print "\nbeginning of %s" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    if configFileName != "":
        config = ConfigParser.ConfigParser()
        config.readfp( open(configFileName) )
        host = config.get("repet_env","repet_host")
        user = config.get("repet_env","repet_user")
        passwd = config.get("repet_env","repet_pw")
        dbname = config.get("repet_env","repet_db")

    if host == "" and os.environ.get( "REPET_HOST" ) != "":
        host = os.environ.get( "REPET_HOST" )
    if user == "" and os.environ.get( "REPET_USER" ) != "":
        user = os.environ.get( "REPET_USER" )
    if passwd == "" and os.environ.get( "REPET_PW" ) != "":
        passwd = os.environ.get( "REPET_PW" )
    if dbname == "" and os.environ.get( "REPET_DB" ) != "":
        dbname = os.environ.get( "REPET_DB" )

    if host == "" or user == "" or passwd == "" or dbname == "":
        print "*** Error: missing information about MySQL connection"
        sys.exit(1)

    logFileName = "%s_RemovePathDoublons.log" % ( inTable )
    if os.path.exists( logFileName ):
        os.remove( logFileName )
    handler = logging.FileHandler( logFileName )
    formatter = logging.Formatter("%(asctime)s %(levelname)s: %(message)s")
    handler.setFormatter( formatter )
    logging.getLogger('').addHandler( handler )
    logging.getLogger('').setLevel( logging.DEBUG )
    logging.info( "started" )

    db = pyRepet.sql.RepetDB.RepetDB( user, host, passwd, dbname )
    if not db.exist( inTable ):
        print "*** Error: table %s doesn't exist" % ( inTable )
        logging.error( "table %s not known!!" % ( inTable ) )
        sys.exit(1)

    # create a temporary file
    tmpFileName = "/tmp/" + inTable + ".tmp" + str(os.getpid()) + "-nr"

    # retrieve the name of the contigs
    tableAdaptator = pyRepet.sql.TableAdaptator.TablePathAdaptator( db, inTable )
    lContigs = tableAdaptator.getContig_name()

    # for each contig
    for c in lContigs:

        string = "processing contig '%s'..." % ( c )

        # retrieve its paths
        clist = tableAdaptator.getPathList_from_contig( c )
        string += "\nnb of paths with doublons: %i" % ( len(clist) )

        # remove doublons
        clist = pyRepet.coord.Path.path_list_remove_doublons( clist )
        string += "\nnb of paths without doublons: %i" % ( len(clist) )

        logging.info( string )
        if verbose > 1:
            print string; sys.stdout.flush()

        # write the non-redundant paths into the temporary file
        pyRepet.coord.Path.path_list_write( clist, tmpFileName, "a" )

    if outTable == "":
        outTable = "%s_nr" % ( inTable )
    string = "create destination table '%s'" % ( outTable )
    logging.info( string )
    if verbose > 1:
        print string

    # load the temporary file in a MySQL table
    db.remove_if_exist( outTable, verbose - 1 )
    db.create_path( outTable, tmpFileName )
    os.remove( tmpFileName )   

    db.close()

    logging.info( "finished" )

    if verbose > 0:
        print "%s finished successfully\n" % (sys.argv[0].split("/")[-1])
        sys.stdout.flush()

    return 0

#-----------------------------------------------------------------------------

if __name__ == '__main__':
    main()
