import os
import pyRepetUnit.commons.coord.Align
import pyRepetUnit.align.AlignList
from pyRepetUnit.align.AlignListUtils import *
import pyRepet.seq.Bioseq
import pyRepet.seq.BioseqUtils
from pyRepet.util.file.FileUtils import *
from pyRepet.coord.Range import *
import pyRepetUnit.commons.IDataProcessor

class TransformAACoordIntoNtCoordInAlignFormat ( pyRepetUnit.commons.IDataProcessor.IDataProcessor ):
    
    def __init__(self):
        self._inFileName = None
        self._clean = False
        self._outFileName = None
        self._consensusFileName = None
        self._IsFiltered = True

    def _getOriginalQueryNameForAlignInstance(self, alignInstance):
        seqName = alignInstance.range_query.seqname[0:len(alignInstance.range_query.seqname) - 2]
        return seqName


    def _invertedSubjectCoord(self, alignInstance):
        return alignInstance.range_subject.reverse()


    def _changeEndInAAIntoNtInPositiveFrame(self, alignInstance, frame, previousEnd):
        alignInstance.range_query.end = 3 * previousEnd + frame - 1

    def _changeStartInAAIntoNtInPositiveFrame(self, alignInstance, frame, previousStart):
        alignInstance.range_query.start = 3 * (previousStart - 1) + frame
    
    def _changeEndInAAIntoNtInNegativeFrame(self, alignInstance, frame, consensusLength, previousStart):
        alignInstance.range_query.end = consensusLength - 3 * (previousStart - 1) - frame + 4

    def _changeStartInAAIntoNtInNegativeFrame(self, alignInstance, frame, consensusLength, previousEnd):
        alignInstance.range_query.start = consensusLength - 3 * (previousEnd - 1) - frame + 2

    def _extractFrameFromSeqName(self, alignInstance):
        frame = int(alignInstance.range_query.seqname[len(alignInstance.range_query.seqname) - 1])
        return frame

    def run(self):   
        alignUtils = AlignListUtils()         
        listAlignInstance = alignUtils.read(self._inFileName)
        self.transformQueryCoord(listAlignInstance, self._consensusFileName)
        self.getOriginalQueryNameForAlignList(listAlignInstance)
        if self._IsFiltered:
            alignUtils.filterOnAMinimalScore(listAlignInstance, 0)
        alignUtils.write(listAlignInstance, self._outFileName)
        if self._clean:
            self.clean()
        
    
    def transformQueryCoord(self, listAlignInstance, DNASeqFastaFilename):
        BioseqUtils = pyRepet.seq.BioseqUtils.BioseqUtils()
        bioseqList = BioseqUtils.extractBioseqListFromFastaFile( DNASeqFastaFilename )
        for alignInstance in listAlignInstance.getList():
            frame = self._extractFrameFromSeqName(alignInstance)
            previousEnd = alignInstance.range_query.end                            
            previousStart = alignInstance.range_query.start
            if (frame == 1) or (frame == 2) or (frame == 3):
                self._changeStartInAAIntoNtInPositiveFrame(alignInstance, frame, previousStart) 
                self._changeEndInAAIntoNtInPositiveFrame(alignInstance, frame, previousEnd)                
            else:
                seqName = self._getOriginalQueryNameForAlignInstance(alignInstance)
                consensusLength = BioseqUtils.getSeqLengthWithSeqName(bioseqList, seqName)              
                self._changeStartInAAIntoNtInNegativeFrame(alignInstance, frame, consensusLength, previousEnd)
                self._changeEndInAAIntoNtInNegativeFrame(alignInstance, frame, consensusLength, previousStart)
                self._invertedSubjectCoord(alignInstance)
    
    def getOriginalQueryNameForAlignList (self, listAlignInstance):
        for alignInstance in listAlignInstance.getList():
            originalSeqName = self._getOriginalQueryNameForAlignInstance(alignInstance)
            alignInstance.range_query.seqname =  originalSeqName   
    
                
    def clean(self):
        os.remove(self._inFileName)
        
    def setInFileName(self, fileName):  
        self._inFileName = fileName  
        
    def setOutFileName(self, fileName):  
        self._outFileName = fileName    
        
    def setConsensusFileName(self, fileName):  
        self._consensusFileName = fileName     
    
    def setIsClean(self, clean):
        self._clean = clean
        
    def getInFileName(self):
        return self._inFileName
    
    def setIsFiltered(self, isFiltered):
        self._IsFiltered = isFiltered
    