import ConfigParser
from ConfigParser import NoOptionError
from pyRepetUnit.commons.checker.IChecker import IChecker
from pyRepetUnit.commons.checker.ConfigException import ConfigException

## A checker for a config file
#
#
# A config file is formatted as follow:
#
# [section1]
#
# option_name1: option_value1
# 
# option_name2: option_value2
#
# option_name3: option_value3
#
# [section2]
# 
# ...
#
# 
# This class performs 3 checkes on a config file: 
#
# (i) check if file exists
#
# (ii) check if section exists
#
# (iii) check if option exists
#
class ConfigChecker (IChecker):

    ## Constructor A checker for config file.
    #
    # @param  sectionName name of section to check in config file
    # @param  optionsDict dictionary with option(s) to check as keys and empty strings ("") as values
    def __init__ (self, sectionName, optionsDict):
        self._sectionName = sectionName
        self._optionsDict = optionsDict

    ## Perform 3 checks : file exists, sections exists, option exists
    # 
    # @param configFile config file to check
    # @exception ConfigException with a list of messages
    def check (self, configFile):
        config = ConfigParser.ConfigParser()
        msg = []
        try:
            config.readfp( open(configFile) )
        except IOError, e:
            msg.append("CONFIG FILE not found - " + e.message)
            raise ConfigException("", msg) 

        if not (config.has_section(self._sectionName)):
            msg.append("[" + self._sectionName + "]" + " section not found - ")
            raise ConfigException("", msg)
         
        isExceptionOccured = False        
        for key in self._optionsDict.keys():
            try:
                self._optionsDict[key] = config.get(self._sectionName, key) 
            except NoOptionError, e:
                msg.append("[" + self._sectionName + "]" + " - " + e.message)
                isExceptionOccured = True
        
        if (isExceptionOccured):
            raise ConfigException("", msg)
